module Model.Position where

import Char
import List
import String

type alias Rank     = Int
type alias File     = Int
type alias Position = (File, Rank)

all : List Position
all = enumFrom minBound

file : File -> List Position
file f = List.map (\r -> (f, r)) [1..8]

rank : Rank -> List Position
rank r = List.map (\f -> (f, r)) [a..h]

isValid : (Int, Int) -> Bool
isValid (r, f) = a <= r && r <= h && 1 <= f && f <= 8


-- File constants --------------------------------------------------------------

a = 1
b = 2
c = 3
d = 4
e = 5
f = 6
g = 7
h = 8

-- Field constants -------------------------------------------------------------

a1 = (a, 1)
a2 = (a, 2)
a3 = (a, 3)
a4 = (a, 4)
a5 = (a, 5)
a6 = (a, 6)
a7 = (a, 7)
a8 = (a, 8)

b1 = (b, 1)
b2 = (b, 2)
b3 = (b, 3)
b4 = (b, 4)
b5 = (b, 5)
b6 = (b, 6)
b7 = (b, 7)
b8 = (b, 8)

c1 = (c, 1)
c2 = (c, 2)
c3 = (c, 3)
c4 = (c, 4)
c5 = (c, 5)
c6 = (c, 6)
c7 = (c, 7)
c8 = (c, 8)

d1 = (d, 1)
d2 = (d, 2)
d3 = (d, 3)
d4 = (d, 4)
d5 = (d, 5)
d6 = (d, 6)
d7 = (d, 7)
d8 = (d, 8)

e1 = (e, 1)
e2 = (e, 2)
e3 = (e, 3)
e4 = (e, 4)
e5 = (e, 5)
e6 = (e, 6)
e7 = (e, 7)
e8 = (e, 8)

f1 = (f, 1)
f2 = (f, 2)
f3 = (f, 3)
f4 = (f, 4)
f5 = (f, 5)
f6 = (f, 6)
f7 = (f, 7)
f8 = (f, 8)

g1 = (g, 1)
g2 = (g, 2)
g3 = (g, 3)
g4 = (g, 4)
g5 = (g, 5)
g6 = (g, 6)
g7 = (g, 7)
g8 = (g, 8)

h1 = (h, 1)
h2 = (h, 2)
h3 = (h, 3)
h4 = (h, 4)
h5 = (h, 5)
h6 = (h, 6)
h7 = (h, 7)
h8 = (h, 8)

-- Bounded ---------------------------------------------------------------------

minBound : Position
minBound = (1, 1)

maxBound : Position
maxBound = (8, 8)


-- Enum ------------------------------------------------------------------------

toEnum : Int -> Position
toEnum i = (i // 8 + 1, i % 8 + 1)

fromEnum : Position -> Int
fromEnum = \(r, f) -> (r - 1) * 8 + (f - 1)

succ : Position -> Position
succ f = toEnum <| fromEnum f + 1

pred : Position -> Position
pred f = toEnum <| fromEnum f - 1

enumFrom : Position -> List Position
enumFrom f1 = enumFromThenTo f1 (succ f1) maxBound

enumFromThen : Position -> Position -> List Position
enumFromThen f1 f2 = enumFromThenTo f1 f2 maxBound

enumFromTo : Position -> Position -> List Position
enumFromTo f1 f2 = enumFromThenTo f1 (succ f1) f2

enumFromThenTo : Position -> Position -> Position -> List Position
enumFromThenTo f1 f2 f3 =
    let start = fromEnum f1
        step  = fromEnum f2 - fromEnum f1
        end   = fromEnum f3
        count = (end - start) // step
    in List.map (\i -> toEnum <| start + i * step) [0..count]


-- Show ------------------------------------------------------------------------

show : Position -> String
show = \(r, f) -> Char.fromCode (96 + r) `String.cons` toString f
